@@ -1,7 +1,7 @@
 require 'nkf'
 def act_text_link(action, group, kz)
   statusbar_timeout = 3 * 1000
-  statusbar_id = kz.statusbar.get_context_id(action.name)
+  name = action.name
   return unless kz.current_page
   uri = kz.current_page.selection_string
   normalized_uri = NKF.nkf("-m0 -Z -Z1 -W -w", uri).gsub(/\s/, '')
@@ -13,13 +13,12 @@ def act_text_link(action, group, kz)
     normalized_uri = normalized_uri.sub(/^[^:]+(s?)/, "http\\1")
     matched = true
   else
-    kz.statusbar.pop(statusbar_id)
-    kz.statusbar.push(statusbar_id, _("couldn't open %s as URI") % uri)
-    Kz.pop_statusbar(kz, statusbar_id, statusbar_timeout)
+    kz.statusbar.set_text(_("Couldn't open %s as URI") % uri, name)
+    Kz.pop_statusbar(kz, name, statusbar_timeout)
   end
 
   if matched
-    kz.statusbar.pop(statusbar_id)
+    kz.statusbar.set_text(nil, name)
     new_tab = Kz::Conf["Global", "entry_open_in_new_tab", Kz::Conf::BOOL]
     if new_tab
       kz.open_new_tab(normalized_uri)
@@ -30,5 +29,5 @@ def act_text_link(action, group, kz)
 end
 def act_text_link_config(config, kz)
   config[:label] = N_("Text Link")
-  config[:tooltip] = N_("open selected text as URI")
+  config[:tooltip] = N_("Open selected text as URI")
 end
