@@ -85,17 +85,9 @@ rb_kz_window_close_all_tab(VALUE self)
 }
 
 static VALUE
-rb_kz_window_reorder_tab(VALUE self, VALUE widget, VALUE position)
+rb_kz_window_move_tab(VALUE self, VALUE dest, VALUE widget)
 {
-    kz_window_reorder_tab(_SELF(self), RVAL2GTKWIDGET(widget),
-                          NUM2INT(position));
-    return self;
-}
-
-static VALUE
-rb_kz_window_move_tab(VALUE self, VALUE widget)
-{
-    kz_window_move_tab(_SELF(self), RVAL2GTKWIDGET(widget));
+    kz_window_move_tab(_SELF(self), RVAL2KZWIN(dest), RVAL2GTKWIDGET(widget));
     return self;
 }
 
@@ -305,9 +297,7 @@ _kz_rb_window_init(VALUE mKz)
     rb_define_method(cKzWindow, "close_tab", rb_kz_window_close_tab, 1);
     rb_define_method(cKzWindow, "close_all_tab",
                      rb_kz_window_close_all_tab, 0);
-    rb_define_method(cKzWindow, "reorder_tab",
-                     rb_kz_window_reorder_tab, 2);
-    rb_define_method(cKzWindow, "move_tab", rb_kz_window_move_tab, 1);
+    rb_define_method(cKzWindow, "move_tab", rb_kz_window_move_tab, 2);
     rb_define_method(cKzWindow, "load_url", rb_kz_window_load_url, 1);
     rb_define_method(cKzWindow, "store_state",
                      rb_kz_window_store_state, 0);
