@@ -2022,16 +2022,15 @@ KzMozWrapper::ShowPageCertificate (void)
 
 #ifdef MOZ_NSICANVASRENDERINGCONTEXTINTERNAL_HAVE_GETINPUTSTREAM_
 nsresult
-KzMozWrapper::CreateThumbnail (void)
+KzMozWrapper::CreateThumbnail (const gchar *uri,
+			       guint last_modified,
+			       EggPixbufThumbSize size)
 {
 	nsCOMPtr<nsIDOMWindow> domWindow;
 	GetDOMWindow(getter_AddRefs(domWindow));
-	
-	nsEmbedCString sURI;
-	GetDocumentUrl(sURI);
-	const gchar *uri = sURI.get();
-	
-	if (!MozillaPrivate::CreateThumbnail(domWindow, uri))
+
+	if (!MozillaPrivate::CreateThumbnail(domWindow, uri,
+					     last_modified, size))
 		return NS_ERROR_FAILURE;
 
 	return NS_OK;
