@@ -105,9 +105,7 @@ act_close_window (GtkAction *action, KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	kz_session_freeze(KZ_SESSION(KZ_GET_CURRENT_SESSION));
 	gtk_widget_destroy(GTK_WIDGET(kz));
-	kz_session_thaw(KZ_SESSION(KZ_GET_CURRENT_SESSION));
 }
 
 
@@ -127,7 +125,7 @@ act_new_window (GtkAction *action, KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	widget = kz_app_create_new_window(kz_app, NULL);
+	widget = KZ_CREATE_NEW_WINDOW(NULL);
 	gtk_widget_show(widget);
 }
 
@@ -297,6 +295,7 @@ act_open_selected_text (GtkAction *action, KzWindow *kz)
 	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
 	gchar *text;
 	GRegex *regex;
+	GMatchInfo *match_info;
 	gint i = 1;
 	gboolean match;
 	GError *error = NULL;
@@ -318,12 +317,12 @@ act_open_selected_text (GtkAction *action, KzWindow *kz)
 	text = kz_embed_get_selection_string(KZ_EMBED(widget));
 	if (!text) return;
 
-	match = g_regex_match(regex, text, 0);
+	match = g_regex_match(regex, text, 0, &match_info);
 	
 	if (match)
 	{
 		gchar **uri;
-		uri = g_regex_fetch_all(regex, text);
+		uri = g_match_info_fetch_all(match_info);
 
 		while (uri[i])
 		{
@@ -336,7 +335,8 @@ act_open_selected_text (GtkAction *action, KzWindow *kz)
 			g_strfreev(uri);
 	}
 
-	g_regex_free(regex);
+	g_match_info_free(match_info);
+	g_regex_unref(regex);
 	g_free(text);
 }
 
@@ -1353,14 +1353,14 @@ act_extract_selected_links (GtkAction *action, KzWindow *kz)
 static void
 act_save_session (GtkAction *action, KzWindow *kz)
 {
-	kz_app_save_session(kz_app);
+	KZ_SAVE_SESSION;
 }
 
 
 static void
 act_restore_session (GtkAction *action, KzWindow *kz)
 {
-	kz_app_restore_session(kz_app);
+	KZ_RESTORE_SESSION;
 }
 
 
@@ -1803,7 +1803,7 @@ GtkActionEntry kz_actions[] =
   {"Cut",                GTK_STOCK_CUT,         N_("C_ut"),           CTRL"X", N_("Cut the selection"), G_CALLBACK(act_cut)},
   {"Copy",               GTK_STOCK_COPY,        N_("_Copy"),          CTRL"C", N_("Copy the selection"), G_CALLBACK(act_copy)},
   {"Paste",              GTK_STOCK_PASTE,       N_("_Paste"),         CTRL"V", N_("Paste the clipboard"), G_CALLBACK(act_paste)},
-  {"SelectAll", KZ_STOCK_SELECT_ALL,            N_("Select _All"),    CTRL"A", N_("Select the entire page"), G_CALLBACK(act_select_all)},
+  {"SelectAll",		 GTK_STOCK_SELECT_ALL,  N_("Select _All"),    CTRL"A", N_("Select the entire page"), G_CALLBACK(act_select_all)},
   {"CopyTitle",          NULL          ,        N_("Copy _Title"),    NULL,    N_("Copy the title of the current page"), G_CALLBACK(act_copy_title)},
   {"CopyURL",            KZ_STOCK_COPY_URL,     N_("Copy _URL"),      NULL,    N_("Copy the URL of the current page"), G_CALLBACK(act_copy_url)},
   {"CopyInUserFormat",          NULL          ,        N_("Copy _In User Format"),    NULL,    N_("Copy Title, URI and Selected text in user format"), NULL}, 
