@@ -42,7 +42,7 @@ struct _KzLocationEntryAction
 	KzHistoryAction   parent;
 	KzWindow         *kz;
 	KzBookmark       *bookmark;
-	KzBookmark	 *history;
+	GList            *history;
 	gboolean          activating;
 	gboolean          synchronizing;
 };
