@@ -35,7 +35,9 @@
 #include "kz-bookmark-item.h"
 #include "kz-bookmark.h"
 #include "kz-bookmark-file.h"
+#include "kz-bookmark-utils.h"
 #include "kz-actions.h"
+#include "kz-notebook.h"
 #include "utils.h"
 #include "glib-utils.h"
 
@@ -53,16 +55,22 @@ enum {
 };
 
 enum {
+	TARGET_GTK_NOTEBOOK_TAB,
 	TARGET_KAZEHAKASE_BOOKMARKS,
 	TARGET_NETSCAPE_URL,
-	TARGET_TEXT_URI_LIST
+	TARGET_TEXT_URI_LIST,
+	TARGET_TEXT_PLAIN,
+	TARGET_STRING
 };
 
-static GtkTargetEntry url_drag_types [] = 
+static const GtkTargetEntry url_drag_types [] = 
 {
-	{"_KAZEHAKASE_BOOKMARKS", 0, TARGET_KAZEHAKASE_BOOKMARKS},
+	{"GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, TARGET_GTK_NOTEBOOK_TAB},
+	{"_KAZEHAKASE_BOOKMARKS", GTK_TARGET_SAME_APP, TARGET_KAZEHAKASE_BOOKMARKS},
         { "_NETSCAPE_URL",        0, TARGET_NETSCAPE_URL},
-	{ "text/uri-list",        0, TARGET_TEXT_URI_LIST}
+	{ "text/uri-list",        0, TARGET_TEXT_URI_LIST},
+	{ "text/plain",      0, TARGET_TEXT_PLAIN},
+	{ "STRING",          0, TARGET_STRING}
 };
 
 static const guint n_url_drag_types = G_N_ELEMENTS (url_drag_types);
@@ -393,13 +401,11 @@ drag_data_received (GtkWidget *widget,
                     guint time)
 {
 	KzBookmarkBar *bar;
-	KzBookmark *bookmark, *sibling, *parent;
-	gchar **strings = NULL; 
+	KzBookmark *bookmark = NULL, *sibling, *parent;
 	GtkWidget *src_widget;
-	const gchar *title;
-	gchar *utf8_title = NULL;
 	GList *children;
 	gint index;
+	gboolean moved = FALSE, success = FALSE;
 
 	bar = KZ_BOOKMARK_BAR(widget);
 	index = gtk_toolbar_get_drop_index(GTK_TOOLBAR(bar->toolbar),
@@ -411,6 +417,19 @@ drag_data_received (GtkWidget *widget,
 
 	switch (info)
 	{
+	 case TARGET_GTK_NOTEBOOK_TAB:
+	 {
+	 	KzEmbed *kzembed;
+		const gchar *uri, *title;
+		src_widget = gtk_drag_get_source_widget(context);
+		if (!KZ_IS_NOTEBOOK(src_widget))
+			break;
+		kzembed = KZ_EMBED(kz_notebook_get_dragged_page(KZ_NOTEBOOK(src_widget)));
+		uri   = kz_embed_get_location(kzembed);
+		title = kz_embed_get_title(kzembed);
+		bookmark = KZ_BOOKMARK(kz_bookmark_new_with_attrs(title, uri, NULL));
+	 	break;
+	 }
 	 case TARGET_KAZEHAKASE_BOOKMARKS:
 		src_widget = gtk_drag_get_source_widget(context);
 		
@@ -419,60 +438,26 @@ drag_data_received (GtkWidget *widget,
 
 		bookmark = KZ_BOOKMARK_ITEM(src_widget)->bookmark;
 		if (bookmark == sibling)
+		{
+			bookmark = NULL;
 			break;
+		}
 	
 		g_object_ref(bookmark);
-
 		parent = kz_bookmark_get_parent(bookmark);
 		kz_bookmark_remove(parent, bookmark);
-
-		if (sibling)
-		{
-			kz_bookmark_insert_before(bar->folder,
-						  bookmark,
-						  sibling);
-		}
-		else
-		{
-			kz_bookmark_append(bar->folder, bookmark);
-		}
-		kz_bookmark_file_save(KZ_BOOKMARK_FILE(bar->folder));
-		gtk_drag_finish(context, TRUE, TRUE, time);
+		moved = TRUE;
 	 	break;
 	 case TARGET_NETSCAPE_URL:
 	 case TARGET_TEXT_URI_LIST:
-		if (data->length < 0) return;
-		strings = g_strsplit((const gchar*)data->data, "\n", 2);
-		if (!strings) return;
-		if (strings[1] != NULL)
-		{
-			utf8_title = g_locale_to_utf8(strings[1],
-						      strlen(strings[1]),
-						      NULL,
-						      NULL,
-						      NULL);
-		}
-		if (utf8_title && g_utf8_validate(utf8_title, -1, NULL))
-			title = utf8_title;
-		else 
-			title = _("title");
-		
-		/* FIXME! we should also add normal bookmark. */
-		/* Create New KzBookmark */
-		if (strstr(strings[0], "xml") ||
-		    strstr(strings[0], "rss") ||
-		    strstr(strings[0], "rdf"))
-		{
-			bookmark = KZ_BOOKMARK(kz_bookmark_file_new(strings[0],
-							title,
-							NULL));
-		}
-		else
-		{
-			bookmark = kz_bookmark_new_with_attrs(title,
-							      strings[0],
-							      NULL);
-		}
+	 	bookmark = kz_bookmark_create_from_selection_data(data);
+		break;
+	 default:
+		break;
+	}
+
+	if (bookmark)
+	{
 		if (sibling)
 		{
 			kz_bookmark_insert_before(bar->folder,
@@ -483,17 +468,12 @@ drag_data_received (GtkWidget *widget,
 		{
 			kz_bookmark_append(bar->folder, bookmark);
 		}
-		g_strfreev(strings);
-		if (utf8_title)
-			g_free(utf8_title);
 		g_object_unref(bookmark);
 		kz_bookmark_file_save(KZ_BOOKMARK_FILE(bar->folder));
-		gtk_drag_finish(context, TRUE, FALSE, time);
-		break;
-	 default:
-	 	gtk_drag_finish(context, FALSE, FALSE, time);
-		break;
+		success = TRUE;
 	}
+
+	gtk_drag_finish(context, success, moved, time);
 }
 
 static void
