@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-item.c,v 1.21 2006/08/06 15:55:49 furyo Exp $
+ *  $Id: kz-bookmark-item.c 3268 2007-10-04 21:54:50Z ikezoe $
  */
 
 #include "kz-bookmark-item.h"
@@ -26,13 +26,13 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include "gobject-utils.h"
 #include "kazehakase.h"
 #include "kz-bookmark-file.h"
 #include "kz-bookmark-menu.h"
 #include "kz-bookmark-menu-item.h"
 #include "kz-bookmark-folder-menu-item.h"
 #include "kz-bookmark-editor.h"
+#include "kz-bookmark-utils.h"
 #include "kz-icons.h"
 #include "kz-actions.h"
 #include "gtk-utils.h"
@@ -49,11 +49,11 @@ enum {
 	TARGET_TEXT_URI_LIST
 };
 
-static GtkTargetEntry dnd_types[] = {
+static const GtkTargetEntry dnd_types[] = {
 	{"_KAZEHAKASE_BOOKMARKS", 0, TARGET_KAZEHAKASE_BOOKMARKS},
 };
 
-static GtkTargetEntry url_drag_types [] = 
+static const GtkTargetEntry url_drag_types [] = 
 {
 	{"_KAZEHAKASE_BOOKMARKS", 0, TARGET_KAZEHAKASE_BOOKMARKS},
         { "_NETSCAPE_URL",        0, TARGET_NETSCAPE_URL},
@@ -63,35 +63,32 @@ static GtkTargetEntry url_drag_types [] =
 static const gint dnd_types_num = G_N_ELEMENTS(dnd_types);
 static const gint n_url_drag_types = G_N_ELEMENTS(url_drag_types);
 
-static void     kz_bookmark_item_class_init       (KzBookmarkItemClass *klass);
-static void     kz_bookmark_item_init             (KzBookmarkItem *menu);
-
 /* object class methods */
-static GObject* kz_bookmark_item_constructor      (GType           type,
-						   guint           n_props,
-						   GObjectConstructParam *props);
-static void     kz_bookmark_item_dispose          (GObject        *object);
-static void     kz_bookmark_item_set_property     (GObject        *object,
-						   guint           prop_id,
-						   const GValue   *value,
-						   GParamSpec     *pspec);
-static void     kz_bookmark_item_get_property     (GObject        *object,
-						   guint           prop_id,
-						   GValue         *value,
-						   GParamSpec     *pspec);
+static GObject *constructor      (GType           type,
+                                  guint           n_props,
+                                  GObjectConstructParam *props);
+static void     dispose          (GObject        *object);
+static void     set_property     (GObject        *object,
+                                  guint           prop_id,
+                                  const GValue   *value,
+                                  GParamSpec     *pspec);
+static void     get_property     (GObject        *object,
+                                  guint           prop_id,
+                                  GValue         *value,
+                                  GParamSpec     *pspec);
 /* GtkToolItem class methods */
-static gboolean kz_bookmark_item_create_menu_proxy (GtkToolItem     *item);
+static gboolean create_menu_proxy (GtkToolItem     *item);
 
 /* widget class methods */
-static void     kz_bookmark_item_drag_data_get    (GtkWidget        *widget,
-						   GdkDragContext   *context,
-						   GtkSelectionData *seldata,
-						   guint             info,
-						   guint             time);
-static void     kz_bookmark_item_drag_begin       (GtkWidget        *widget,
-						   GdkDragContext   *context);
-static void     kz_bookmark_item_drag_end         (GtkWidget        *widget,
-						   GdkDragContext   *context);
+static void    drag_data_get    (GtkWidget        *widget,
+                                 GdkDragContext   *context,
+                                 GtkSelectionData *seldata,
+                                 guint             info,
+                                 guint             time);
+static void    drag_begin       (GtkWidget        *widget,
+                                 GdkDragContext   *context);
+static void    drag_end         (GtkWidget        *widget,
+                                 GdkDragContext   *context);
 
 /* private methods */
 static void     kz_bookmark_item_reset_submenu    (KzBookmarkItem *menu);
@@ -123,14 +120,7 @@ static void     cb_drag_data_received             (GtkWidget        *widget,
 						   guint             info,
 						   guint             time,
 						   gpointer          data);
-
-
-static GtkToolItemClass *parent_class = NULL;
-
-
-KZ_OBJECT_GET_TYPE(kz_bookmark_item, "KzBookmarkItem", KzBookmarkItem,
-		   kz_bookmark_item_class_init, kz_bookmark_item_init,
-		   GTK_TYPE_TOOL_ITEM)
+G_DEFINE_TYPE(KzBookmarkItem, kz_bookmark_item, GTK_TYPE_TOOL_ITEM)
 
 static void
 kz_bookmark_item_class_init (KzBookmarkItemClass *klass)
@@ -139,23 +129,21 @@ kz_bookmark_item_class_init (KzBookmarkItemClass *klass)
 	GtkWidgetClass *widget_class;
 	GtkToolItemClass *tool_item_class;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class   = (GObjectClass *) klass;
 	widget_class    = (GtkWidgetClass *) klass;
 	tool_item_class = (GtkToolItemClass *) klass;
 
 	/* GObject signals */
-	gobject_class->dispose      = kz_bookmark_item_dispose;
-	gobject_class->constructor  = kz_bookmark_item_constructor;
-	gobject_class->set_property = kz_bookmark_item_set_property;
-	gobject_class->get_property = kz_bookmark_item_get_property;
+	gobject_class->dispose      = dispose;
+	gobject_class->constructor  = constructor;
+	gobject_class->set_property = set_property;
+	gobject_class->get_property = get_property;
 
-	widget_class->drag_data_get = kz_bookmark_item_drag_data_get;
-	widget_class->drag_begin    = kz_bookmark_item_drag_begin;
-	widget_class->drag_end      = kz_bookmark_item_drag_end;
+	widget_class->drag_data_get = drag_data_get;
+	widget_class->drag_begin    = drag_begin;
+	widget_class->drag_end      = drag_end;
 
-  	tool_item_class->create_menu_proxy = kz_bookmark_item_create_menu_proxy;
+  	tool_item_class->create_menu_proxy = create_menu_proxy;
 
 	g_object_class_install_property
 		(gobject_class,
@@ -190,16 +178,16 @@ kz_bookmark_item_init (KzBookmarkItem *menu)
 
 
 static GObject*
-kz_bookmark_item_constructor (GType                  type,
-			      guint                  n_props,
-			      GObjectConstructParam *props)
+constructor (GType                  type,
+             guint                  n_props,
+             GObjectConstructParam *props)
 {
 	KzBookmarkItem *item;
 	GtkWidget *button, *image, *eventbox, *hbox;
 	GtkContainer *container;
 	GtkRcStyle *style;
 	GObject *object;
-	GObjectClass *klass = G_OBJECT_CLASS(parent_class);
+	GObjectClass *klass = G_OBJECT_CLASS(kz_bookmark_item_parent_class);
 	KzBookmarkFileState state;
 	gboolean is_file, is_folder;
 
@@ -246,16 +234,13 @@ kz_bookmark_item_constructor (GType                  type,
 		}
 		else
 		{
-			KzFavicon *kzfav;
+			KzFavicon *kzfav = KZ_GET_FAVICON;
 			GtkWidget *favicon = NULL;
 
 			/* favicon */
-			/* kzfav = item->kz->kzfav; */
-			kzfav = kz_favicon_get_instance();
 			favicon	= kz_favicon_get_widget(kzfav,
 							kz_bookmark_get_link(item->bookmark),
 							KZ_ICON_SIZE_BOOKMARK_MENU);
-			g_object_unref(kzfav);
 			if (favicon)
 			{
 				item->image = image = favicon;
@@ -275,7 +260,7 @@ kz_bookmark_item_constructor (GType                  type,
 	style = gtk_rc_style_new ();
 	style->xthickness = style->ythickness = 0;
 	gtk_widget_modify_style (button, style);
-	gtk_rc_style_unref (style);
+	g_object_unref (style);
 
 	if (is_file)
 	{
@@ -328,12 +313,12 @@ kz_bookmark_item_constructor (GType                  type,
 
 
 static void
-kz_bookmark_item_dispose (GObject *object)
+dispose (GObject *object)
 {
 	KzBookmarkItem *menu;
 
-	if (G_OBJECT_CLASS (parent_class)->dispose)
-		G_OBJECT_CLASS (parent_class)->dispose(object);
+	if (G_OBJECT_CLASS (kz_bookmark_item_parent_class)->dispose)
+		G_OBJECT_CLASS (kz_bookmark_item_parent_class)->dispose(object);
 
 	menu = KZ_BOOKMARK_ITEM(object);
 
@@ -374,7 +359,7 @@ kz_bookmark_item_dispose (GObject *object)
 
 
 static void
-kz_bookmark_item_set_property (GObject         *object,
+set_property (GObject         *object,
 			       guint            prop_id,
 			       const GValue    *value,
 			       GParamSpec      *pspec)
@@ -397,7 +382,7 @@ kz_bookmark_item_set_property (GObject         *object,
 
 
 static void
-kz_bookmark_item_get_property (GObject         *object,
+get_property (GObject         *object,
 			       guint            prop_id,
 			       GValue          *value,
 			       GParamSpec      *pspec)
@@ -420,7 +405,7 @@ kz_bookmark_item_get_property (GObject         *object,
 
 
 static gboolean
-kz_bookmark_item_create_menu_proxy (GtkToolItem *item)
+create_menu_proxy (GtkToolItem *item)
 {
 	KzBookmarkItem *bookmark_item = KZ_BOOKMARK_ITEM(item);
 	GtkWidget *proxy;
@@ -617,11 +602,11 @@ cb_update_button_clicked (GtkWidget *widget, KzBookmarkItem *menu)
 
 
 static void
-kz_bookmark_item_drag_data_get (GtkWidget *widget,
-				GdkDragContext *context,
-				GtkSelectionData *seldata,
-				guint info,
-				guint time)
+drag_data_get (GtkWidget *widget,
+               GdkDragContext *context,
+               GtkSelectionData *seldata,
+               guint info,
+               guint time)
 {
 	switch (info)
 	{
@@ -634,7 +619,7 @@ kz_bookmark_item_drag_data_get (GtkWidget *widget,
 
 
 static void
-kz_bookmark_item_drag_begin (GtkWidget *widget, GdkDragContext *context)
+drag_begin (GtkWidget *widget, GdkDragContext *context)
 {
 	KzBookmarkItem *item;
 	KzBookmark *bookmark;
@@ -656,7 +641,7 @@ kz_bookmark_item_drag_begin (GtkWidget *widget, GdkDragContext *context)
 
 
 static void
-kz_bookmark_item_drag_end (GtkWidget *widget, GdkDragContext *context)
+drag_end (GtkWidget *widget, GdkDragContext *context)
 {
 	gtk_widget_show(widget);
 }
@@ -673,11 +658,9 @@ cb_drag_data_received (GtkWidget        *widget,
 		       gpointer          data)
 {
 	KzBookmarkItem *item;
-	KzBookmark *bookmark, *parent, *file;
-	gchar **strings = NULL; 
+	KzBookmark *bookmark = NULL, *parent, *file;
 	GtkWidget *src_widget;
-	const gchar *title;
-	gchar *utf8_title = NULL;
+	gboolean success = FALSE, moved = FALSE;
 
 	item = KZ_BOOKMARK_ITEM(widget);
 	switch (info)
@@ -690,93 +673,41 @@ cb_drag_data_received (GtkWidget        *widget,
 
 		bookmark = KZ_BOOKMARK_ITEM(src_widget)->bookmark;
 		g_object_ref(bookmark);
-
 		parent = kz_bookmark_get_parent(bookmark);
 		kz_bookmark_remove(parent, bookmark);
-
-		kz_bookmark_append(item->bookmark, bookmark);
-		if (KZ_IS_BOOKMARK_FILE(item->bookmark))
-			file = item->bookmark;
-		else
-			file = kz_bookmark_get_parent_file(item->bookmark);
-		
-		if (kz_bookmark_file_has_xmlrpc(KZ_BOOKMARK_FILE(file)))
-		{
-			kz_bookmark_file_xmlrpc_insert(KZ_BOOKMARK_FILE(file),
-						       item->bookmark,
-						       NULL,
-						       bookmark);
-		}
-		else
-		{
-			kz_bookmark_file_save(KZ_BOOKMARK_FILE(file));
-		}
-
-		gtk_drag_finish(context, TRUE, TRUE, time);
+		moved = TRUE;
 	 	break;
 	 case TARGET_NETSCAPE_URL:
 	 case TARGET_TEXT_URI_LIST:
-		if (selection_data->length < 0) return;
-		strings = g_strsplit((const gchar*)selection_data->data, "\n", 2);
-		if (!strings) return;
-		if (strings[1] != NULL)
-		{
-			utf8_title = g_locale_to_utf8(strings[1],
-						      strlen(strings[1]),
-						      NULL,
-						      NULL,
-						      NULL);
-		}
-		if (utf8_title && g_utf8_validate(utf8_title, -1, NULL))
-			title = utf8_title;
-		else 
-			title = _("Title");
-		
-		/* FIXME! we should also add normal bookmark. */
-		/* Create New KzBookmark */
-		if (strstr(strings[0], "xml") ||
-		    strstr(strings[0], "rss") ||
-		    strstr(strings[0], "rdf"))
-		{
-			bookmark = KZ_BOOKMARK(kz_bookmark_file_new(strings[0],
-							title,
-							NULL));
-		}
-		else
-		{
-			bookmark = kz_bookmark_new_with_attrs(title,
-							      strings[0],
-							      NULL);
-		}
-		kz_bookmark_append(item->bookmark, bookmark);
-		g_strfreev(strings);
-		if (utf8_title)
-			g_free(utf8_title);
-		g_object_unref(bookmark);
+	 	bookmark = kz_bookmark_create_from_selection_data(selection_data);
+		break;
+	 default:
+		break;
+	}
 
+	if (bookmark)
+	{
+		kz_bookmark_append(item->bookmark, bookmark);
 		if (KZ_IS_BOOKMARK_FILE(item->bookmark))
 			file = item->bookmark;
 		else
 			file = kz_bookmark_get_parent_file(item->bookmark);
-		
+
 		if (kz_bookmark_file_has_xmlrpc(KZ_BOOKMARK_FILE(file)))
 		{
 			kz_bookmark_file_xmlrpc_insert(KZ_BOOKMARK_FILE(file),
-						       item->bookmark,
-						       NULL,
-						       bookmark);
+					item->bookmark,
+					NULL,
+					bookmark);
 		}
 		else
 		{
 			kz_bookmark_file_save(KZ_BOOKMARK_FILE(file));
 		}
-	
-		gtk_drag_finish(context, TRUE, FALSE, time);
-		break;
-	 default:
-	 	gtk_drag_finish(context, FALSE, FALSE, time);
-		break;
+		g_object_unref(bookmark);
+		success = TRUE;
 	}
+	gtk_drag_finish(context, success, moved, time);
 }
 
 
