@@ -73,7 +73,7 @@ kz_rss_is_supported(KzBookmarkFile *bookmark, const gchar *buf)
 
 	g_return_val_if_fail(buf, FALSE);
 
-	if (!g_str_has_prefix(buf, "<?xml")) return FALSE;
+	/* if (!g_str_has_prefix(buf, "<?xml")) return FALSE; */
 
 	pos = buf;
 
