@@ -84,18 +84,9 @@ struct _KzWindowPrivate
 	
 	/* sidebar */
 	gboolean sidebar_was_shown;
-	GNode *tab_tree;
 };
 #define KZ_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_WINDOW, KzWindowPrivate))
 
-enum {
-	TARGET_KAZEHAKASE_TAB,
-	TARGET_NETSCAPE_URL,
-	TARGET_TEXT_URI_LIST,
-	TARGET_TEXT_PLAIN,
-	TARGET_STRING
-};
-
 static struct {
 	const gchar *pos;
 	const gchar *tab_act;
@@ -116,12 +107,46 @@ static struct {
 	{KZ_SHIFT_KEY,"shift"},
 };
 
+enum {
+	TARGET_NETSCAPE_URL,
+	TARGET_TEXT_URI_LIST
+};
+
+static const GtkTargetEntry drag_targets [] = 
+{
+        { "_NETSCAPE_URL",        0, TARGET_NETSCAPE_URL},
+	{ "text/uri-list",        0, TARGET_TEXT_URI_LIST}
+};
+
+static const guint n_drag_targets = G_N_ELEMENTS (drag_targets);
+
 static gboolean key_theme_is_emacs = FALSE;
 
 static gboolean delete_event   (GtkWidget     *widget,
 				GdkEventAny   *event);
 static gboolean key_press_event(GtkWidget     *widget,
 				GdkEventKey   *event);
+static void     drag_data_received
+			       (GtkWidget *widget,
+			        GdkDragContext *context,
+				gint x, gint y,
+				GtkSelectionData *data,
+				guint info,
+				guint time);
+static gboolean button_release_event
+			       (GtkWidget      *widget,
+			        GdkEventButton *event);
+static gboolean scroll_event   (GtkWidget      *widget,
+				GdkEventScroll *event);
+/*
+ * mozilla doesn't accept these signals, so we connect these funtions to
+ * KzWindow instead of KzEmbed.
+ */
+static gboolean motion_notify_event
+			       (GtkWidget      *widget,
+				GdkEventMotion *event);
+static gboolean focus_out_event(GtkWidget       *widget,
+			        GdkEventFocus   *event);
 static void     destroy        (GtkObject     *object);
 
 static void     kz_window_append_tab     (KzWindow      *kz,
@@ -182,6 +207,10 @@ static void     cb_menu_merge_add_widget      (GtkUIManager    *merge,
 static void     cb_clipboard_get_text         (GtkClipboard *clipboard,
 					       const gchar *text,
 					       gpointer data);
+static void     cb_notebook_reorder_page      (GtkNotebook     *notebook,
+					       GtkWidget       *child,
+					       guint            page_num,
+					       KzWindow        *kz);
 static void     cb_notebook_switch_page       (GtkNotebook     *notebook,
 					       GtkNotebookPage *page,
 					       guint            page_num,
@@ -197,9 +226,6 @@ static void     cb_sidebar_map                (GtkWidget       *widget,
 					       GtkToggleAction *action);
 static void     cb_sidebar_unmap              (GtkWidget       *widget,
 					       GtkToggleAction *action);
-static gboolean cb_focus_out_event            (GtkWidget       *widget,
-					       GdkEventFocus   *event,
-					       KzWindow        *kz);
  
 /* callbacks for embed */
 static void     cb_embed_title_changed         (KzEmbed     *embed,
@@ -237,19 +263,6 @@ static gint     cb_embed_dom_mouse_over          (KzEmbed *embed,
 						KzEmbedEventMouse *event,
 						KzWindow    *kz);
 
-/*
- * mozilla doesn't accept these signals, so we connect these funtions to
- * KzWindow instead of KzEmbed.
- */
-static gboolean cb_window_motion_notify_event  (GtkWidget      *widget,
-						GdkEventMotion *event,
-						KzEmbed        *kzembed);
-static gboolean cb_window_button_release_event (GtkWidget      *widget,
-						GdkEventButton *event,
-						KzEmbed        *kzembed);
-static gboolean cb_window_scroll_event         (GtkWidget      *widget,
-						GdkEventScroll *event,
-						KzEmbed        *kzembed);
 
 static void     gtk_key_theme_changed_cb       (GtkSettings  *settings,
                                                 GParamSpec   *pspec,
@@ -310,8 +323,13 @@ kz_window_class_init (KzWindowClass *klass)
 	object_class->destroy = destroy;
 
 	/* GtkWidget signals */
-	widget_class->delete_event    = delete_event;
-	widget_class->key_press_event = key_press_event;
+	widget_class->delete_event         = delete_event;
+	widget_class->key_press_event      = key_press_event;
+	widget_class->drag_data_received   = drag_data_received;
+	widget_class->button_release_event = button_release_event;
+	widget_class->scroll_event         = scroll_event;
+	widget_class->motion_notify_event  = motion_notify_event;
+	widget_class->focus_out_event      = focus_out_event;
 
 	/* KzWindow signals */
 	klass->append_tab = kz_window_append_tab;
@@ -416,8 +434,6 @@ kz_window_init (KzWindow *kz)
 	
 	priv->sidebar_was_shown = FALSE;
 
-	priv->tab_tree     = NULL;
-
 	kz->tabs            = kz_bookmark_pure_folder_new();
 	kz->closed_tabs     = kz_bookmark_pure_folder_new();
 	kz->history_search  = kz_bookmark_pure_folder_new();
@@ -435,9 +451,6 @@ kz_window_init (KzWindow *kz)
 		g_signal_connect(KZ_GET_GLOBAL_PROFILE,
 				 "changed::Gesture",
 				 G_CALLBACK(cb_profile_gesture_changed), kz);
-		g_signal_connect(kz,
-				 "focus-out-event",
-				 G_CALLBACK(cb_focus_out_event), kz);
 	}
 
 	/* top level vbox */
@@ -537,6 +550,8 @@ kz_window_init (KzWindow *kz)
 
 	/* main notebook widget */
 	{
+		g_signal_connect(kz->notebook, "page-reordered",
+				 G_CALLBACK(cb_notebook_reorder_page), kz);
 		g_signal_connect(kz->notebook, "switch-page", 
 				 G_CALLBACK(cb_notebook_switch_page), kz);
 		g_signal_connect_after(kz->notebook, "switch-page",
@@ -567,6 +582,11 @@ kz_window_init (KzWindow *kz)
 		gtk_widget_show(kz->statusbar);
 	}
 
+	gtk_drag_dest_set(GTK_WIDGET(kz), 
+			  GTK_DEST_DEFAULT_ALL, 
+                          drag_targets, n_drag_targets,
+			  GDK_ACTION_MOVE | GDK_ACTION_COPY);
+
 	/* from Galeon-1.3.18 */
 	/* initialize the listener for the key theme */
 	settings = gtk_settings_get_default();
@@ -577,11 +597,6 @@ kz_window_init (KzWindow *kz)
 	gtk_key_theme_changed_cb(settings, 0, 0);
 
 	kz_bookmark_set_title(kz->tabs, "Window");
-	if (!kz_session_is_frozen(KZ_SESSION(KZ_GET_CURRENT_SESSION))) {
-		/* FIXME!: Session should block inserting new bookmark by it
-		   self. */
-		kz_bookmark_append(KZ_GET_CURRENT_SESSION, kz->tabs);
-	}
 
 	kz_window_restore_state(kz);
 }
@@ -810,17 +825,10 @@ kz_window_create_embed (KzWindow *kz)
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 
-	engine = KZ_CONF_GET_STR("Global", "layout_engine");
+	engine = KZ_CONF_GET_STR("Browser", "layout_engine");
 	if (engine)
 	{
-		if (!strcmp(engine, "gtk+-webcore"))
-		{
-			kz_embed = KZ_EMBED(kz_embed_new("gtk-webcore"));
-		}
-		else
-		{
-			kz_embed = KZ_EMBED(kz_embed_new("gecko"));
-		}
+		kz_embed = KZ_EMBED(kz_embed_new(engine));
 		g_free(engine);
 	}
 
@@ -828,6 +836,8 @@ kz_window_create_embed (KzWindow *kz)
 		kz_embed = KZ_EMBED(kz_embed_new("gecko"));
 	if (!kz_embed)
 		kz_embed = KZ_EMBED(kz_embed_new("gtk-webcore"));
+	if (!kz_embed)
+		kz_embed = KZ_EMBED(kz_embed_new("ie"));
 
 	return kz_embed;
 }
@@ -839,41 +849,25 @@ kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 {
 	KzEmbed *kzembed;
 	KzTabLabel *kztab, *sibtab;
-	GNode *node, *parent_node;
-	KzWindowPrivate *priv;
 	
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 	if (url) g_return_val_if_fail(g_utf8_validate(url, strlen(url), NULL), NULL);
 
-	priv = KZ_WINDOW_GET_PRIVATE(kz);
 	kzembed = kz_window_create_embed(kz);
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
 
 	gtk_widget_show_all(GTK_WIDGET(kzembed));
 	gtk_widget_show(GTK_WIDGET(kztab));
 
-	sibtab = kz_notebook_open_new_tab(KZ_NOTEBOOK(kz->notebook), kzembed, kztab);
+	kz_notebook_open_new_tab(KZ_NOTEBOOK(kz->notebook), kzembed, kztab);
+	sibtab = kz_notebook_get_sibling_tab_label(KZ_NOTEBOOK(kz->notebook), kztab);
 	/* insert tab bookmark */
-	if(sibtab)
-		kz_bookmark_insert_before(kz->tabs, 
-					  kztab->history,
-					  sibtab->history);
+	kz_bookmark_insert_before(kz->tabs, 
+				  kztab->history,
+				  sibtab ? sibtab->history : NULL);
 
 	kz_window_set_embed_callbacks(kz, kzembed);
 
-	/* root node */
-	if (!priv->tab_tree)
-		priv->tab_tree = g_node_new(NULL);
-
-	/* insret node */
-	node = g_node_new(kzembed);
-	parent_node = g_node_find(priv->tab_tree,
-				  G_IN_ORDER, G_TRAVERSE_ALL, parent);
-	if (parent_node)
-		g_node_append(parent_node, node);
-	else
-		g_node_append(priv->tab_tree, node);
-
 	g_signal_emit(kz, kz_window_signals[APPEND_TAB_SIGNAL],
 		      0, kzembed, parent);
 
@@ -895,7 +889,6 @@ kz_window_restore_tabs (KzWindow *kz, KzBookmark *tabs)
 	priv = KZ_WINDOW_GET_PRIVATE(kz);
 
 	/* remove old tab bookmark */
-	kz_bookmark_remove(KZ_GET_CURRENT_SESSION, kz->tabs);
 	g_object_unref(kz->tabs);
 	kz->tabs = g_object_ref(tabs);
 
@@ -905,7 +898,6 @@ kz_window_restore_tabs (KzWindow *kz, KzBookmark *tabs)
 	{
 		KzEmbed *kzembed;
 		KzTabLabel *kztab;
-		GNode *node;
 		KzBookmark *child;
 		kzembed = kz_window_create_embed(kz);
 		kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
@@ -913,18 +905,9 @@ kz_window_restore_tabs (KzWindow *kz, KzBookmark *tabs)
 
 		gtk_widget_show_all(GTK_WIDGET(kzembed));
 		gtk_widget_show(GTK_WIDGET(kztab));
-		kz_notebook_open_new_tab_at_pos(KZ_NOTEBOOK(kz->notebook), kzembed, kztab, -1);
+		kz_notebook_prepend_new_tab(KZ_NOTEBOOK(kz->notebook), kzembed, kztab);
 		kz_window_set_embed_callbacks(kz, kzembed);
 		kz_tab_label_set_history(kztab, child);
-
-		/* root node */
-		if (!priv->tab_tree)
-			priv->tab_tree = g_node_new(NULL);
-
-		/* insret node */
-		node = g_node_new(kzembed);
-		g_node_append(priv->tab_tree, node);
-
 	}
 	g_list_free(childtabs);
 
@@ -959,111 +942,77 @@ kz_window_close_all_tab(KzWindow *kz)
 }
 
 
-void
-kz_window_reorder_tab (KzWindow *kz, GtkWidget *widget, gint pos)
-{
-	KzTabLabel *kztab, *sibtab;
-	KzBookmark *sib_bookmark = NULL;
-
-	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(GTK_IS_WIDGET(widget));
-
-	sibtab = KZ_TAB_LABEL(kz_notebook_reorder_tab(KZ_NOTEBOOK(kz->notebook), widget, pos));
-
-	if (sibtab)
-		sib_bookmark = sibtab->history;
-
-	kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook), widget));
-	g_object_ref(kztab->history);
-	kz_session_freeze(KZ_SESSION(KZ_GET_CURRENT_SESSION));
-	kz_bookmark_remove(kz->tabs, kztab->history);
-	kz_bookmark_insert_before(kz->tabs, 
-				  kztab->history,
-				  sib_bookmark);
-	kz_session_thaw(KZ_SESSION(KZ_GET_CURRENT_SESSION));
-	g_object_unref(kztab->history);
-
-	g_signal_emit(kz, kz_window_signals[REORDER_TAB_SIGNAL],
-		      0, widget, pos);
-}
-
 /* FIXME!! It's a hacky way. */
 void
-kz_window_move_tab (KzWindow *kz, GtkWidget *widget)
+kz_window_move_tab (KzWindow *src_kz, KzWindow *dest_kz, GtkWidget *widget)
 {
-	GtkNotebook *src_notebook;
+	GtkNotebook *src_notebook, *dest_notebook;
 	GtkWidget *label;
-	KzWindow *src_kz;
 	KzEmbed *kzembed;
 	KzTabLabel *new_kztab;
-	GNode *node, *child;
-	KzWindowPrivate *priv, *src_priv;
 
-	g_return_if_fail(KZ_IS_WINDOW(kz));
+	g_return_if_fail(KZ_IS_WINDOW(src_kz));
+	g_return_if_fail(KZ_IS_WINDOW(dest_kz));
 	g_return_if_fail(KZ_IS_EMBED(widget));
 
-	priv = KZ_WINDOW_GET_PRIVATE(kz);
+	src_notebook = GTK_NOTEBOOK(src_kz->notebook);
+	dest_notebook = GTK_NOTEBOOK(dest_kz->notebook);
+
 	kzembed = KZ_EMBED(widget);
 
-	src_kz = KZ_WINDOW(kz_app_get_window_from_tab(kz_app, widget));
-	if (!src_kz) return;
-
-	src_priv = KZ_WINDOW_GET_PRIVATE(src_kz);
-	label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
-					   widget);
+	label = gtk_notebook_get_tab_label(dest_notebook, widget);
 
 	/* the kzembed is already the kz's child */
 	if (label) return;
 
-	src_notebook = GTK_NOTEBOOK(src_kz->notebook);
-	label = gtk_notebook_get_tab_label(src_notebook,
-					   widget);
+	label = gtk_notebook_get_tab_label(src_notebook, widget);
 	g_return_if_fail(label);
 
 	/* create new tab label */
-	new_kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
+	new_kztab = KZ_TAB_LABEL(kz_tab_label_new(dest_kz, kzembed));
 
 	/* move the page to this window */
 	kz_window_unset_cur_embed_callbacks(src_kz, kzembed);
 	kz_window_unset_embed_callbacks(src_kz, kzembed);
 
-	kz_notebook_move_tab(KZ_NOTEBOOK(kz->notebook), KZ_NOTEBOOK(src_kz->notebook), widget);
+	kz_notebook_move_tab(KZ_NOTEBOOK(src_notebook), KZ_NOTEBOOK(dest_notebook), widget);
 
-	kz_bookmark_prepend(kz->tabs, new_kztab->history);
-	kz_window_set_embed_callbacks(kz, kzembed);
+	kz_bookmark_prepend(dest_kz->tabs, new_kztab->history);
+	kz_window_set_embed_callbacks(dest_kz, kzembed);
 
-	/* move tab tree */
-	node = g_node_find(src_priv->tab_tree,
-			   G_IN_ORDER, G_TRAVERSE_ALL, widget);
-	if (node)
+	g_signal_emit(src_kz,
+		      kz_window_signals[REMOVE_TAB_SIGNAL],
+		      0, kzembed);
+	g_signal_emit(dest_kz,
+		      kz_window_signals[APPEND_TAB_SIGNAL],
+		      0, kzembed, NULL);
+}
+
+
+static void
+drag_data_received (GtkWidget *widget,
+                    GdkDragContext *context,
+                    gint x, gint y,
+                    GtkSelectionData *seldata,
+                    guint info,
+                    guint time)
+{
+	switch (info)
 	{
-		/* move children */
-		child = g_node_first_child(node);
-		while (child)
-		{
-			GNode *next = g_node_next_sibling(child);
-			g_node_unlink(child);
-			g_node_append(src_priv->tab_tree,
-				      child);
-			child = next;
-		}
+	 case TARGET_NETSCAPE_URL:
+	 case TARGET_TEXT_URI_LIST:
+	 {
+		KzWindow *kz = KZ_WINDOW(widget);
+		gchar **strings;
 
-		/* move node */
-		g_node_unlink(node);
-		if (!priv->tab_tree)
-			priv->tab_tree = g_node_new(NULL);
-		g_node_append(priv->tab_tree, node);
-		
-		g_signal_emit(src_kz,
-			      kz_window_signals[REMOVE_TAB_SIGNAL],
-			      0, kzembed);
-		g_signal_emit(kz,
-			      kz_window_signals[APPEND_TAB_SIGNAL],
-			      0, kzembed, NULL);
-	}
-	else
-	{
-		g_warning("KzWindow: cannot find tab node!");
+		if (seldata->length < 0) return;
+
+		strings = g_strsplit((const gchar*)seldata->data, "\n", 2);
+		kz_window_open_new_tab(kz, strings[0]);
+		g_strfreev(strings);
+		gtk_drag_finish(context, TRUE, FALSE, time);
+		break;
+	 }
 	}
 }
 
@@ -1077,9 +1026,9 @@ delete_event (GtkWidget *widget, GdkEventAny *event)
 
 	kz_window_store_state(kz);
 
-	kz_session_freeze(KZ_SESSION(KZ_GET_CURRENT_SESSION));
+	KZ_FREEZE_SESSION;
 	kz_notebook_close_all_tab(KZ_NOTEBOOK(kz->notebook));
-	kz_session_thaw(KZ_SESSION(KZ_GET_CURRENT_SESSION));
+	KZ_THAW_SESSION;
 
 	return FALSE;
 }
@@ -1214,16 +1163,14 @@ destroy (GtkObject *object)
 	kz->popup_tablist = NULL;
 
 	if (kz->tabs)
-	{
-		kz_session_freeze(KZ_SESSION(KZ_GET_CURRENT_SESSION));
-		kz_bookmark_remove(KZ_GET_CURRENT_SESSION, kz->tabs);
+		/* CAUTION: make KzApp sure to remove tabs before freeing it. */
 		g_object_unref(kz->tabs);
-		kz_session_thaw(KZ_SESSION(KZ_GET_CURRENT_SESSION));
-		kz->tabs = NULL;
-	}
+	kz->tabs = NULL;
+
 	if (kz->closed_tabs)
 		g_object_unref(kz->closed_tabs);
 	kz->closed_tabs = NULL;
+
 	if (kz->history_search)
 		g_object_unref(kz->history_search);
 	kz->history_search = NULL;
@@ -1499,16 +1446,6 @@ kz_window_set_cur_embed_callbacks(KzWindow *kz, KzEmbed *kzembed)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
 
-	g_signal_connect(kz,
-			 "motion_notify_event",
-			 G_CALLBACK(cb_window_motion_notify_event), kzembed);
-	g_signal_connect(kz,
-			 "button_release_event",
-			 G_CALLBACK(cb_window_button_release_event), kzembed);
-	g_signal_connect(kz,
-			 "scroll_event",
-			 G_CALLBACK(cb_window_scroll_event), kzembed);
-
 	/*
 	 * key event signal
 	 */
@@ -1552,16 +1489,6 @@ kz_window_unset_cur_embed_callbacks(KzWindow *kz, KzEmbed *kzembed)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
 
-	g_signal_handlers_disconnect_by_func
-		(kz,
-		 G_CALLBACK(cb_window_motion_notify_event), kzembed);
-	g_signal_handlers_disconnect_by_func
-		(kz,
-		 G_CALLBACK(cb_window_button_release_event), kzembed);
-	g_signal_handlers_disconnect_by_func
-		(kz,
-		 G_CALLBACK(cb_window_scroll_event), kzembed);
-
 	/*
 	 * key event signal
 	 */
@@ -1709,7 +1636,7 @@ GNode *
 kz_window_get_tree (KzWindow *kz)
 {
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
-	return KZ_WINDOW_GET_PRIVATE(kz)->tab_tree;
+	return kz_notebook_get_tree(KZ_NOTEBOOK(kz->notebook));
 }
 
 void
@@ -1981,6 +1908,15 @@ cb_clipboard_get_text(GtkClipboard *clipboard, const gchar *text, gpointer data)
 }
 
 
+static void
+cb_notebook_reorder_page (GtkNotebook *notebook, GtkWidget *child,
+			  guint page_num, KzWindow *kz)
+{
+	g_signal_emit(kz, kz_window_signals[REORDER_TAB_SIGNAL],
+		      0, child, page_num);
+
+}
+
 static void
 cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 			 guint page_num, KzWindow *kz)
@@ -2035,7 +1971,9 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	kz_actions_set_tab_sensitive(kz, kzembed);
 
 	/* set current_position in history */
-	kz_bookmark_set_current(kz->tabs, page_num);
+	/* FIXME: KzSession should block automatically */
+	if (!kz_session_is_frozen(KZ_SESSION(KZ_GET_CURRENT_SESSION)))
+		kz_bookmark_set_current(kz->tabs, page_num);
 }
 
 static void
@@ -2185,12 +2123,8 @@ static void
 cb_embed_close_tab (GtkObject *obj, KzWindow *kz)
 {
 	KzEmbed *kzembed;
-	GNode *node, *child;
-	KzWindowPrivate *priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	priv = KZ_WINDOW_GET_PRIVATE(kz);
-	g_return_if_fail(priv->tab_tree);
 
 	kzembed = KZ_EMBED(obj);
 
@@ -2200,22 +2134,6 @@ cb_embed_close_tab (GtkObject *obj, KzWindow *kz)
 	kz_notebook_close_tab(KZ_NOTEBOOK(kz->notebook), GTK_WIDGET(obj));
 	kz_statusbar_set_link_text(KZ_STATUSBAR(kz->statusbar), NULL);
 
-	node = g_node_find(priv->tab_tree,
-			   G_IN_ORDER, G_TRAVERSE_ALL, kzembed);
-	/* probably destroy handler is already called */
-	if (!node) return;
-
-	/* move children */
-	child = g_node_first_child(node);
-	while (child)
-	{
-		GNode *next = g_node_next_sibling(child);
-		g_node_unlink(child);
-		g_node_append(priv->tab_tree, child);
-		child = next;
-	}
-	
-	g_node_destroy(node);
 	g_signal_emit(kz, kz_window_signals[REMOVE_TAB_SIGNAL],
 		      0, kzembed);
 }
@@ -2479,16 +2397,14 @@ cb_embed_dom_mouse_over (KzEmbed *embed, KzEmbedEventMouse *event,
 }
 
 static gboolean
-cb_window_motion_notify_event (GtkWidget *widget,
-			       GdkEventMotion *event,
-			       KzEmbed *kzembed)
+motion_notify_event (GtkWidget *widget,
+		     GdkEventMotion *event)
 {
 	KzWindow *kz;
 	gint x, y;
 	/* GdkModifierType state; */
 	KzWindowPrivate *priv;
-
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
+	KzEmbed *kzembed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(KZ_WINDOW(widget)));
 
 	kz = KZ_WINDOW(widget);
 	priv = KZ_WINDOW_GET_PRIVATE(kz);
@@ -2514,16 +2430,13 @@ cb_window_motion_notify_event (GtkWidget *widget,
 
 
 static gboolean
-cb_window_button_release_event (GtkWidget *widget,
-				GdkEventButton *event,
-				KzEmbed *kzembed)
+button_release_event (GtkWidget *widget,
+		      GdkEventButton *event)
 {
 	KzWindow *kz;
 	gboolean retval = FALSE;
 	KzWindowPrivate *priv;
 
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-
 	kz = KZ_WINDOW(widget);
 	priv = KZ_WINDOW_GET_PRIVATE(kz);
 
@@ -2569,16 +2482,13 @@ cb_window_button_release_event (GtkWidget *widget,
 }
 
 static gboolean
-cb_window_scroll_event (GtkWidget *widget,
-			GdkEventScroll *event,
-			KzEmbed *kzembed)
+scroll_event (GtkWidget *widget,
+	      GdkEventScroll *event)
 {
 	KzWindow *kz;
 	gboolean retval = FALSE;
 	KzWindowPrivate *priv;
 
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-
 	kz = KZ_WINDOW(widget);
 	priv = KZ_WINDOW_GET_PRIVATE(kz);
 
@@ -2592,13 +2502,12 @@ cb_window_scroll_event (GtkWidget *widget,
 }
 
 static gboolean 
-cb_focus_out_event (GtkWidget *widget,
-		    GdkEventFocus *event,
-		    KzWindow *kz)
+focus_out_event (GtkWidget *widget,
+		 GdkEventFocus *event)
 {
 	if(!event->in) 
 	{
-		KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE(kz);
+		KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE(widget);
 		/* focus is out */
 		if (kz_gesture_is_started(priv->gesture))
 		{
@@ -2607,7 +2516,7 @@ cb_focus_out_event (GtkWidget *widget,
 			{
 				gdk_pointer_ungrab(gtk_get_current_event_time());
 			}
-			kz_statusbar_set_gesture_text(KZ_STATUSBAR(kz->statusbar), NULL);
+			kz_statusbar_set_gesture_text(KZ_STATUSBAR(KZ_WINDOW(widget)->statusbar), NULL);
 		}
 	}
 	return FALSE;
